part of '../bolter_provider.dart';

mixin _PresenterProviderSingleChildWidget on SingleChildWidget {}

class PresenterProvider<P extends Presenter> extends SingleChildStatelessWidget
    with _PresenterProviderSingleChildWidget {
  final P presenter;
  final bool lazy;

  const PresenterProvider(
      {Key key,
      @required Widget child,
      @required this.presenter,
      this.lazy = false})
      : super(key: key, child: child);

  static P of<P extends Presenter>(BuildContext context) =>
      Provider.of<P>(context, listen: false);

  @override
  Widget buildWithChild(BuildContext context, Widget child) {
    final bolterProvider = BolterProvider.of(context);
    return Provider<P>(
      lazy: lazy,
      create: (_) {
        presenter?._bolter = bolterProvider?._bolter;
        presenter?._flutterState = bolterProvider?._flutterState;
        presenter?._useCaseContainer = bolterProvider?._useCaseContainer;
        presenter?._context = context;
        presenter?.init();
        return presenter;
      },
      dispose: (_, presenter) {
        presenter?.dispose();
      },
      child: child,
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider(
      {Key key, @required this.providers, @required this.child})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter>() => PresenterProvider.of<P>(this);
}
